#include <proto.h>

#include "gtmodview.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtModView class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init modview.
*
\************************************************************************/
GtModView *GtMkModView(char *n,GtManager *p,GtTag *tgs)
{
	GtModView *obj;
	
	if(!(obj=new GtModView()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtModView object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init modview.
*
\************************************************************************/
GtModView *GtMkModView(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtModView *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkModView(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}
	
/************************************************************************\
*
* Setup empty modview.
*
\************************************************************************/
GtModView::GtModView(void)
{
	maxchild=4;
	vs[0]=vs[1]=vs[2]=vs[3]=NULL;
	vp[0]=0;
	vp[1]=1;
	vp[2]=2;
	vp[3]=3;
	ibw=0;
	space=0;
}

/************************************************************************\
*
* Setup modview.
*
\************************************************************************/
sbool GtModView::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtBdrMan::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_IBDR,GTRT_BYTE,&ibw);
	app->GetRes(this,GTR_SPACE,GTRT_BYTE,&space);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_InnerBorder:
				ibw=t->d.val;
				break;
			case GT_Spacing:
				space=t->d.val;
				break;
		}
	}
	return(TRUE);
}

/************************************************************************\
*
* Set modview attributes.
*
\************************************************************************/
sbool GtModView::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE;
	
	if(!GtBdrMan::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_InnerBorder:
				ibw=t->d.val;
				ch=TRUE;
				break;
			case GT_Spacing:
				space=t->d.val;
				ch=TRUE;
				break;
		}
	}
	if(ch)
		this->Relayout();
	return(TRUE);
}

/************************************************************************\
*
* Get modview attributes.
*
\************************************************************************/
void GtModView::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtBdrMan::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_InnerBorder:
				*ptr=ibw;
				break;
			case GT_Spacing:
				*ptr=space;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtModView::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtModView")==0)
		return(TRUE);
	else
		return(GtBdrMan::IsSubClassOf(cn));
}

/************************************************************************\
*
* Relayout modview with title.
*
\************************************************************************/
void GtModView::Relayout(void)
{
	GtWListBdr *li;
	GtWidget *obj=NULL;
	int n,xs,ys;
	word mxs=1,mys=1,mxxs=GTS_MAX,mxys=GTS_MAX;
	
	if(flags & GTWF_UNBUILT)
		return;
	n=0;
	for(li=(GtWListBdr *)GtSFirst(&man); li && (n<4);
			li=(GtWListBdr *)GtSNext(&li->wl.lnk))
	{
		vs[n]=li;
		obj=li->wl.w;
		obj->GetMinSize(&xs,&ys);
		xs=GTMAX(xs,li->minxs);
		ys=GTMAX(ys,li->minys);
		mxs=GTMAX(mxs,2*ibw+space+2*(xs+li->bleft+li->bright));
		mys=GTMAX(mys,2*ibw+space+2*(ys+li->btop+li->bbot));
		obj->GetMaxSize(&xs,&ys);
		xs=GTMIN(xs,li->maxxs);
		ys=GTMIN(ys,li->maxys);
		mxxs=GTMIN(mxxs,2*ibw+space+2*(xs+li->bleft+li->bright));
		mxys=GTMIN(mxys,2*ibw+space+2*(ys+li->btop+li->bbot));
		n++;
	}
	this->SetMinMaxSize(GTMAX(mxs,2*ibw+1),GTMAX(mys,2*ibw+1),mxxs,mxys);
}

/************************************************************************\
*
* Set modview layout.
*
\************************************************************************/
void GtModView::SetView(int ul,int ur,int ll,int lr)
{
	int n;
	sbool mv[4]={FALSE};
	ulong tf;
	
	mv[ul]=TRUE;
	mv[ur]=TRUE;
	mv[ll]=TRUE;
	mv[lr]=TRUE;
	vp[GTMV_UPLEFT]=ul;
	vp[GTMV_UPRIGHT]=ur;
	vp[GTMV_DOWNLEFT]=ll;
	vp[GTMV_DOWNRIGHT]=lr;
	tf=flags;
	flags|=GTWF_UNBUILT;
	for(n=0; n<4; n++)
	{
		if(vs[n])
			vs[n]->wl.w->Map(mv[n]);
	}
	flags=tf;
	this->Layout();
}

/************************************************************************\
*
* Get modview layout.
*
\************************************************************************/
void GtModView::GetView(int *ul,int *ur,int *ll,int *lr)
{
	if(ul)
		*ul = vp[GTMV_UPLEFT];
	if(ur)
		*ur = vp[GTMV_UPRIGHT];
	if(ll)
		*ll = vp[GTMV_DOWNLEFT];
	if(lr)
		*lr = vp[GTMV_DOWNRIGHT];
}

/************************************************************************\
*
* Layout modview and title.
*
\************************************************************************/
void GtModView::Layout(void)
{
	GtWListBdr *li;
	int xpe=0,ype=0,xp,yp,xs,ys;
	
	if(flags & GTWF_UNBUILT)
		return;
	if(flags & GTWF_NOWIN)
	{
		xpe=xpos;
		ype=ypos;
	}
	if(vs[GTMV_UPLEFT] && (vp[GTMV_UPLEFT]==GTMV_UPLEFT))
	{
		li=vs[GTMV_UPLEFT];
		xp=0;
		yp=0;
		xs=(xsize-2*ibw-space)/2;
		ys=(ysize-2*ibw-space)/2;
		if(vp[GTMV_UPRIGHT]==GTMV_UPLEFT)
			xs=xsize-2*ibw;
		if(vp[GTMV_DOWNLEFT]==GTMV_UPLEFT)
			ys=ysize-2*ibw;
		li->wl.w->SetPosSize(ibw+xpe+xp+li->bleft,ibw+ype+yp+li->bright,
			xs-li->bleft-li->bright,ys-li->btop-li->bbot);
	}
	if(vs[GTMV_UPRIGHT] && (vp[GTMV_UPRIGHT]==GTMV_UPRIGHT))
	{
		li=vs[GTMV_UPRIGHT];
		xs=(xsize-2*ibw-space)/2;
		ys=(ysize-2*ibw-space)/2;
		xp=xs+space;
		yp=0;
		if(vp[GTMV_UPLEFT]==GTMV_UPRIGHT)
		{
			xs=xsize-2*ibw;
			xp=0;
		}
		if(vp[GTMV_DOWNRIGHT]==GTMV_UPRIGHT)
			ys=ysize-2*ibw;
		li->wl.w->SetPosSize(ibw+xpe+xp+li->bleft,ibw+ype+yp+li->bright,
			xs-li->bleft-li->bright,ys-li->btop-li->bbot);
	}
	if(vs[GTMV_DOWNLEFT] && (vp[GTMV_DOWNLEFT]==GTMV_DOWNLEFT))
	{
		li=vs[GTMV_DOWNLEFT];
		xs=(xsize-2*ibw-space)/2;
		ys=(ysize-2*ibw-space)/2;
		xp=0;
		yp=ys+space;
		if(vp[GTMV_DOWNRIGHT]==GTMV_DOWNLEFT)
			xs=xsize-2*ibw;
		if(vp[GTMV_UPLEFT]==GTMV_DOWNLEFT)
		{
			ys=ysize-2*ibw;
			yp=0;
		}
		li->wl.w->SetPosSize(ibw+xpe+xp+li->bleft,ibw+ype+yp+li->bright,
			xs-li->bleft-li->bright,ys-li->btop-li->bbot);
	}
	if(vs[GTMV_DOWNRIGHT] && (vp[GTMV_DOWNRIGHT]==GTMV_DOWNRIGHT))
	{
		li=vs[GTMV_DOWNRIGHT];
		xs=(xsize-2*ibw-space)/2;
		ys=(ysize-2*ibw-space)/2;
		xp=xs+space;
		yp=ys+space;
		if(vp[GTMV_DOWNLEFT]==GTMV_DOWNRIGHT)
		{
			xs=xsize-2*ibw;
			xp=0;
		}
		if(vp[GTMV_UPRIGHT]==GTMV_DOWNRIGHT)
		{
			ys=ysize-2*ibw;
			yp=0;
		}
		li->wl.w->SetPosSize(ibw+xpe+xp+li->bleft,ibw+ype+yp+li->bright,
			xs-li->bleft-li->bright,ys-li->btop-li->bbot);
	}
}
